@@ -69,63 +69,28 @@
 -export([binding/1,binding_exists/1,
          dir_listing/1, dir_listing/2, redirect_self/1]).
 
--export([arg_clisock/1
-         , arg_client_ip_port/1
-         , arg_headers/1
-         , arg_req/1
-         , arg_clidata/1
-         , arg_server_path/1
-         , arg_querydata/1
-         , arg_appmoddata/1
-         , arg_docroot/1
-         , arg_docroot_mount/1
-         , arg_fullpath/1
-         , arg_cont/1
-         , arg_state/1
-         , arg_pid/1
-         , arg_opaque/1
-         , arg_appmod_prepath/1
-         , arg_prepath/1
-         , arg_pathinfo/1
+-export([arg_clisock/1, arg_client_ip_port/1, arg_headers/1, arg_req/1,
+         arg_clidata/1, arg_server_path/1, arg_querydata/1, arg_appmoddata/1,
+         arg_docroot/1, arg_docroot_mount/1, arg_fullpath/1, arg_cont/1,
+         arg_state/1, arg_pid/1, arg_opaque/1, arg_appmod_prepath/1, arg_prepath/1,
+         arg_pathinfo/1]).
+-export([http_request_method/1, http_request_path/1, http_request_version/1,
+         http_response_version/1, http_response_status/1, http_response_phrase/1,
+         headers_connection/1, headers_accept/1, headers_host/1,
+         headers_if_modified_since/1, headers_if_match/1, headers_if_none_match/1,
+         headers_if_range/1, headers_if_unmodified_since/1, headers_range/1,
+         headers_referer/1, headers_user_agent/1, headers_accept_ranges/1,
+         headers_cookie/1, headers_keep_alive/1, headers_location/1,
+         headers_content_length/1, headers_content_type/1,
+         headers_content_encoding/1, headers_authorization/1,
+         headers_transfer_encoding/1, headers_x_forwarded_for/1, headers_other/1]).
 
-         , http_request_method/1
-         , http_request_path/1
-         , http_request_version/1
+-export([set_header/2, set_header/3, get_header/2, get_header/3, delete_header/2]).
 
-         , http_response_version/1
-         , http_response_status/1
-         , http_response_phrase/1
+-import(lists, [flatten/1, reverse/1]).
 
-         , headers_connection/1
-         , headers_accept/1
-         , headers_host/1
-         , headers_if_modified_since/1
-         , headers_if_match/1
-         , headers_if_none_match/1
-         , headers_if_range/1
-         , headers_if_unmodified_since/1
-         , headers_range/1
-         , headers_referer/1
-         , headers_user_agent/1
-         , headers_accept_ranges/1
-         , headers_cookie/1
-         , headers_keep_alive/1
-         , headers_location/1
-         , headers_content_length/1
-         , headers_content_type/1
-         , headers_content_encoding/1
-         , headers_authorization/1
-         , headers_transfer_encoding/1
-         , headers_x_forwarded_for/1
-         , headers_other/1
-
-        ]).
-
-
--import(lists, [map/2, flatten/1, reverse/1]).
-
-%% these are a bunch of function that are useful inside
-%% yaws scripts
+%% These are a bunch of accessor functions that are useful inside
+%% yaws scripts.
 
 arg_clisock(#arg{clisock = X}) -> X.
 arg_client_ip_port(#arg{client_ip_port = X}) -> X.
@@ -228,9 +193,11 @@ parse_post(Arg) ->
 %% should return {get_more, Cont, User_state} where User_state might
 %% usefully be a File Descriptor.
 %%
-%% or {result, Res} if this is the last (or only) segment.
+%% {result, Res} if this is the last (or only) segment.
 %%
-%% Res is a list of {header, Header} | {part_body, Binary} | {body, Binary}
+%% or {error, Reason} if an error occurred during the parsing.
+%%
+%% Res is a list of {head, {Name, Hdrs}} | {part_body, Binary} | {body, Binary}
 %%
 %% Example usage could be:
 %%
@@ -243,10 +210,12 @@ parse_post(Arg) ->
 %%                    {get_more, Cont, St};
 %%             {result, Res} ->
 %%                    handle_res(A, Res),
-%%                    {html, f("<pre>Done </pre>",[])}
+%%                    {html, f("<pre>Done </pre>",[])};
+%%             {error, Reason} ->
+%%                    {html, f("An error occured: ~p", [Reason])}
 %%        end.
 %%
-%% handle_res(A, [{head, Name}|T]) ->
+%% handle_res(A, [{head, {Name, Hdrs}}|T]) ->
 %%      io:format("head:~p~n",[Name]),
 %%      handle_res(A, T);
 %% handle_res(A, [{part_body, Data}|T]) ->
@@ -270,9 +239,7 @@ parse_multipart_post(Arg, Options) ->
         'POST' ->
             case CT of
                 undefined ->
-                    error_logger:error_msg("Can't parse multipart if we "
-                                           "have no Content-Type header",[]),
-                    [];
+                    {error, no_content_type};
                 "multipart/form-data"++Line ->
                     case Arg#arg.cont of
                         {cont, Cont} ->
@@ -282,20 +249,16 @@ parse_multipart_post(Arg, Options) ->
                         undefined ->
                             LineArgs = parse_arg_line(Line),
                             {value, {_, Boundary}} =
-                                lists:keysearch(boundary, 1, LineArgs),
+                                lists:keysearch("boundary", 1, LineArgs),
                             parse_multipart(
                               un_partial(Arg#arg.clidata),
                               Boundary, Options)
                     end;
                 _Other ->
-                    error_logger:error_msg("Can't parse multipart if we "
-                                           "find no multipart/form-data",[]),
-                    []
+                    {error, no_multipart_form_data}
             end;
-        Other ->
-            error_logger:error_msg("Can't parse multipart if get a ~p",
-                                   [Other]),
-            []
+        _Other ->
+            {error, bad_method}
     end.
 
 un_partial({partial, Bin}) ->
@@ -354,7 +317,7 @@ parse_arg_value([C|Line], Key, Value, Quote, _) ->
 %%
 
 make_parse_line_reply(Key, Value, Rest) ->
-    {{list_to_atom(yaws:funreverse(Key, fun yaws:to_lowerchar/1)),
+    {{yaws:funreverse(Key, fun yaws:to_lowerchar/1),
       lists:reverse(Value)}, Rest}.
 
 
@@ -371,128 +334,119 @@ parse_multipart(Data, St) ->
     parse_multipart(Data, St, [list]).
 parse_multipart(Data, St, Options) ->
     case parse_multi(Data, St, Options) of
-        {cont, St2, Res} ->
-            {cont, {cont, St2}, lists:reverse(Res)};
-        {result, Res} ->
-            {result, lists:reverse(Res)}
+        {cont, St2, Res} -> {cont, {cont, St2}, lists:reverse(Res)};
+        {result, Res}    -> {result, lists:reverse(Res)};
+        {error, Reason}  -> {error, Reason}
     end.
 
-%% Reentry point
-parse_multi(Data, {cont, #mp_parse_state{old_data = OldData}=ParseState}, _) ->
+parse_multi(Data, #mp_parse_state{state=boundary}=ParseState, Acc) ->
+    %% Find the beginning of the next part or the last boundary
+    case bm_find(Data, ParseState#mp_parse_state.boundary_ctx) of
+        {Pos, Len} ->
+            %% If Pos != 0, ignore data preceding the boundary
+            case Data of
+                <<_:Pos/binary, Boundary:Len/binary>> ->
+                    %% Not enough data to tell if it is the last boundary or not
+                    {cont, ParseState#mp_parse_state{old_data=Boundary}, Acc};
+                <<_:Pos/binary, _:Len/binary, "\r\n", Rest/binary>> ->
+                    %% It is not the last boundary, so parse the next part
+                    NPState = ParseState#mp_parse_state{state=start_headers},
+                    parse_multi(Rest, NPState, Acc);
+                <<_:Pos/binary, _:Len/binary, "--\r\n", _/binary>> ->
+                    %% Match on the last boundary and ignore remaining data
+                    {result, Acc};
+                _ ->
+                    {error, malformed_multipart_post}
+            end;
+        nomatch ->
+            %% No boundary found, request more data. Here we keep just enough
+            %% data to match on the boundary the next time
+            DLen = size(Data),
+            BLen = bm_length(ParseState#mp_parse_state.boundary_ctx),
+            SkipLen = erlang:max(DLen - BLen, 0),
+            KeepLen = erlang:min(BLen, DLen),
+            <<_:SkipLen/binary, OldData:KeepLen/binary>> = Data,
+            {cont, ParseState#mp_parse_state{old_data=OldData}, Acc}
+    end;
+
+parse_multi(Data, #mp_parse_state{state=start_headers}=ParseState, Acc) ->
+    parse_multi(Data, ParseState, Acc, [], []);
+
+parse_multi(Data, #mp_parse_state{state=body}=ParseState, Acc) ->
+    %% Find the end of this part (i.e the next boundary)
+    case bm_find(Data, ParseState#mp_parse_state.boundary_ctx) of
+        {Pos, _Len} ->
+            %% Extract the body and keep the boundary
+            <<Body:Pos/binary, Rest/binary>> = Data,
+            BodyData = case ParseState#mp_parse_state.data_type of
+                           list   -> binary_to_list(Body);
+                           binary -> Body
+                       end,
+            NAcc = [{body, BodyData}|Acc],
+            NParseState = ParseState#mp_parse_state{state=boundary},
+            parse_multi(Rest, NParseState, NAcc);
+        nomatch ->
+            %% No boundary found, request more data.
+            DLen = size(Data),
+            BLen = bm_length(ParseState#mp_parse_state.boundary_ctx),
+            SkipLen = erlang:max(DLen - BLen, 0),
+            KeepLen = erlang:min(BLen, DLen),
+            <<PartData:SkipLen/binary, OldData:KeepLen/binary>> = Data,
+            NParseState = ParseState#mp_parse_state{state=body,
+                                                    old_data=OldData},
+            BodyData = case ParseState#mp_parse_state.data_type of
+                           list   -> binary_to_list(PartData);
+                           binary -> PartData
+                       end,
+            {cont, NParseState, [{part_body, BodyData}|Acc]}
+    end;
+
+parse_multi(Data, {cont, #mp_parse_state{old_data=OldData}=ParseState}, _) ->
+    %% Reentry point
     NData = <<OldData/binary, Data/binary>>,
     parse_multi(NData, ParseState, []);
 
-parse_multi(<<"--\r\n", Data/binary>>,
-            #mp_parse_state{state=boundary}=ParseState, Acc) ->
-    parse_multi(Data, ParseState, Acc);
-parse_multi(Data, #mp_parse_state{state=boundary}=ParseState, Acc) ->
-    #mp_parse_state{boundary_ctx = BoundaryCtx} = ParseState,
-    case bm_find(Data, BoundaryCtx) of
-        {0, Len} ->
-            LenPlusCRLF = Len+2,
-            <<_:LenPlusCRLF/binary, Rest/binary>> = Data,
-            NParseState = ParseState#mp_parse_state{state = start_header},
-            parse_multi(Rest, NParseState, Acc);
-        {_Pos, _Len} ->
-            {NParseState, NData} = case Data of
-                                       <<"\r\n", Rest/binary>> ->
-                                           {ParseState#mp_parse_state{
-                                              state = start_header},
-                                            Rest};
-                                       _ ->
-                                           {ParseState#mp_parse_state{
-                                              state = body}, Data}
-                                   end,
-            parse_multi(NData, NParseState, Acc);
-        nomatch ->
-            case Data of
-                <<>> ->
-                    {result, Acc};
-                <<"\r\n">> ->
-                    {result, Acc};
-                _ ->
-                    NParseState = ParseState#mp_parse_state{old_data = Data},
-                    {cont, NParseState, Acc}
-            end
-    end;
-parse_multi(Data, #mp_parse_state{state=start_header}=ParseState, Acc) ->
-    NParseState = ParseState#mp_parse_state{state = header},
-    parse_multi(Data, NParseState, Acc, [], []);
-parse_multi(Data, #mp_parse_state{state=body}=ParseState, Acc) ->
-    #mp_parse_state{boundary_ctx = BoundaryCtx} = ParseState,
-    case bm_find(Data, BoundaryCtx) of
-        {Pos, Len} ->
-            <<Body:Pos/binary, _:Len/binary, Rest/binary>> = Data,
-            BodyData = case ParseState#mp_parse_state.data_type of
-                           list ->
-                               binary_to_list(Body);
-                           binary ->
-                               Body
-                       end,
-            NAcc = [{body, BodyData}|Acc],
-            NParseState = ParseState#mp_parse_state{state = boundary},
-            parse_multi(Rest, NParseState, NAcc);
-        nomatch ->
-            NParseState = ParseState#mp_parse_state{
-                            state = body,
-                            old_data = <<>>
-                           },
-            BodyData = case ParseState#mp_parse_state.data_type of
-                           list ->
-                               binary_to_list(Data);
-                           binary ->
-                               Data
-                       end,
-            NAcc = [{part_body, BodyData}|Acc],
-            {cont, NParseState, NAcc}
-    end;
-%% Initial entry point
 parse_multi(Data, Boundary, Options) ->
-    B1 = "\r\n--"++Boundary,
-    D1 = <<"\r\n", Data/binary>>,
-    BoundaryCtx = bm_start(B1),
-    HdrEndCtx = bm_start("\r\n\r\n"),
-    DataType = lists:foldl(fun(_, list) ->
-                                   list;
-                              (list, _) ->
-                                   list;
-                              (binary, undefined) ->
-                                   binary;
-                              (_, Acc) ->
-                                   Acc
-                           end, undefined, Options),
-    ParseState = #mp_parse_state{state = boundary,
+    %% Initial entry point
+    BoundaryCtx = bm_start("\r\n--"++Boundary),
+    HdrEndCtx   = bm_start("\r\n\r\n"),
+    DataType    = lists:foldl(fun(_,      list)      -> list;
+                                 (list,   _)         -> list;
+                                 (binary, undefined) -> binary;
+                                 (_,      Acc)       -> Acc
+                              end, undefined, Options),
+    ParseState = #mp_parse_state{state        = boundary,
                                  boundary_ctx = BoundaryCtx,
-                                 hdr_end_ctx = HdrEndCtx,
-                                 data_type = DataType},
-    parse_multi(D1, ParseState, []).
+                                 hdr_end_ctx  = HdrEndCtx,
+                                 data_type    = DataType},
+    parse_multi(<<"\r\n", Data/binary>>, ParseState, []).
 
-parse_multi(Data, #mp_parse_state{state=start_header}=ParseState, Acc, [], []) ->
-    #mp_parse_state{hdr_end_ctx = HdrEndCtx} = ParseState,
-    case bm_find(Data, HdrEndCtx) of
+
+parse_multi(Data, #mp_parse_state{state=start_headers}=ParseState,
+            Acc, [], []) ->
+    %% Find the end of headers for this part
+    case bm_find(Data, ParseState#mp_parse_state.hdr_end_ctx) of
+        {_Pos, _Len} ->
+            %% We have all headers, we can parse it
+            NParseState = ParseState#mp_parse_state{state=headers},
+            parse_multi(Data, NParseState, Acc, [], []);
         nomatch ->
-            {cont, ParseState#mp_parse_state{old_data = Data}, Acc};
-        _ ->
-            NParseState = ParseState#mp_parse_state{state = header},
-            parse_multi(Data, NParseState, Acc, [], [])
+            {cont, ParseState#mp_parse_state{old_data=Data}, Acc}
     end;
-parse_multi(Data, #mp_parse_state{state=header}=ParseState, Acc, Name, Hdrs) ->
+parse_multi(Data, #mp_parse_state{state=headers}=ParseState, Acc, Name, Hdrs) ->
     case erlang:decode_packet(httph_bin, Data, [{packet_size, 16#4000}]) of
         {ok, http_eoh, Rest} ->
+            %% All headers are parsed, get the body now
             Head = case Name of
-                       [] ->
-                           lists:reverse(Hdrs);
-                       _ ->
-                           {Name, lists:reverse(Hdrs)}
+                       [] -> lists:reverse(Hdrs);
+                       _  -> {Name, lists:reverse(Hdrs)}
                    end,
-            NParseState = ParseState#mp_parse_state{state = body},
+            NParseState = ParseState#mp_parse_state{state=body},
             parse_multi(Rest, NParseState, [{head, Head}|Acc]);
         {ok, {http_header, _, Hdr, _, HdrVal}, Rest} when is_atom(Hdr) ->
             Header = {case Hdr of
-                          'Content-Type' ->
-                              content_type;
-                          Else ->
-                              Else
+                          'Content-Type' -> content_type;
+                          Else           -> Else
                       end,
                       binary_to_list(HdrVal)},
             parse_multi(Rest, ParseState, Acc, Name, [Header|Hdrs]);
@@ -502,15 +456,15 @@ parse_multi(Data, #mp_parse_state{state=header}=ParseState, Acc, Name, Hdrs) ->
                 "content-disposition" ->
                     "form-data"++Params = HdrValStr,
                     Parameters = parse_arg_line(Params),
-                    {value, {_, NewName}} = lists:keysearch(name, 1, Parameters),
+                    {_, NewName} = lists:keyfind("name", 1, Parameters),
                     parse_multi(Rest, ParseState, Acc,
                                 NewName, Parameters++Hdrs);
                 LowerHdr ->
                     parse_multi(Rest, ParseState, Acc,
                                 Name, [{LowerHdr, HdrValStr}|Hdrs])
             end;
-        {error, _Reason}=Error ->
-            Error
+        _ ->
+            {error, malformed_multipart_post}
     end.
 
 %% parse POST data when ENCTYPE is unset or
@@ -599,12 +553,16 @@ code_to_phrase(414) -> "Request-URI Too Long";
 code_to_phrase(415) -> "Unsupported Media Type";
 code_to_phrase(416) -> "Requested Range Not Satisfiable";
 code_to_phrase(417) -> "Expectation Failed";
+code_to_phrase(428) -> "Precondition Required";
+code_to_phrase(429) -> "Too Many Requests";
+code_to_phrase(431) -> "Request Header Fields Too Large";
 code_to_phrase(500) -> "Internal Server Error";
 code_to_phrase(501) -> "Not Implemented";
 code_to_phrase(502) -> "Bad Gateway";
 code_to_phrase(503) -> "Service Unavailable";
 code_to_phrase(504) -> "Gateway Timeout";
 code_to_phrase(505) -> "HTTP Version Not Supported";
+code_to_phrase(511) -> "Network Authentication Required";
 
 %% Below are some non-HTTP status codes from other protocol standards that
 %% we've seen used with HTTP in the wild, so we include them here. HTTP 1.1
@@ -745,10 +703,9 @@ setcookie(Name, Value, Path, Expire, Domain, Secure) ->
 %% to search for a specific cookie
 %% return [] if not found
 %%        Str if found
-%% if serveral cookies with the same name are passed fron the browser,
+%% if several cookies with the same name are passed fron the browser,
 %% only the first match is returned
 
-
 find_cookie_val(Cookie, A) when is_record(A, arg) ->
     find_cookie_val(Cookie,  (A#arg.headers)#headers.cookie);
 %%
@@ -777,11 +734,14 @@ eat_cookie(Cookie, Str) when is_list(Cookie),is_list(Str) ->
 %% Look for the Cookie and extract its value.
 eat_cookie2(_, [], _)    ->
     throw("not found");
-eat_cookie2([H|T], [H|R], C) ->
-    eat_cookie2(T, R, C);
-eat_cookie2([H|_], [X|R], C) when H =/= X ->
-    {_,Rest} = eat_until(R, $;),
-    eat_cookie(C, Rest);
+eat_cookie2([H1|T], [H2|R], C) ->
+    case string:to_lower(H1) =:= string:to_lower(H2) of
+        true ->
+            eat_cookie2(T, R, C);
+        false ->
+            {_,Rest} = eat_until(R, $;),
+            eat_cookie(C, Rest)
+    end;
 eat_cookie2([], L, _) ->
     {Meat,_} = eat_until(L, $;),
     Meat.
@@ -1046,7 +1006,7 @@ set_access_log(Bool) ->
 %% interactively turn on|off tracing to the tty (as well)
 %% typically useful in embedded mode
 set_tty_trace(Bool) ->
-    yaws_log:trace_tty(Bool).
+    yaws_trace:set_tty_trace(Bool).
 
 
 
@@ -1148,6 +1108,11 @@ reformat_header(H) ->
                  true ->
                       {"Content-Type", H#headers.content_type}
               end,
+              if H#headers.content_encoding == undefined ->
+                      undefined;
+                 true ->
+                      {"Content-Encoding", H#headers.content_encoding}
+              end,
 
               if H#headers.authorization == undefined ->
                       undefined;
@@ -1163,6 +1128,11 @@ reformat_header(H) ->
                       undefined;
                  true ->
                       {"Location", H#headers.location}
+              end,
+              if H#headers.x_forwarded_for == undefined ->
+                      undefined;
+                 true ->
+                      {"X-Forwarded-For", H#headers.x_forwarded_for}
               end
 
              ]
@@ -1173,6 +1143,293 @@ reformat_header(H) ->
           end, H#headers.other).
 
 
+set_header(#headers{}=Hdrs, {Header, Value}) ->
+    set_header(Hdrs, Header, Value).
+
+set_header(#headers{}=Hdrs, connection, Value) ->
+    Hdrs#headers{connection = Value};
+set_header(#headers{}=Hdrs, {lower, "connection"}, Value) ->
+    Hdrs#headers{connection = Value};
+set_header(#headers{}=Hdrs, accept, Value) ->
+    Hdrs#headers{accept = Value};
+set_header(#headers{}=Hdrs, {lower, "accept"}, Value) ->
+    Hdrs#headers{accept = Value};
+set_header(#headers{}=Hdrs, host, Value) ->
+    Hdrs#headers{host = Value};
+set_header(#headers{}=Hdrs, {lower, "host"}, Value) ->
+    Hdrs#headers{host = Value};
+set_header(#headers{}=Hdrs, if_modified_since, Value) ->
+    Hdrs#headers{if_modified_since = Value};
+set_header(#headers{}=Hdrs, {lower, "if-modified-since"}, Value) ->
+    Hdrs#headers{if_modified_since = Value};
+set_header(#headers{}=Hdrs, if_match, Value) ->
+    Hdrs#headers{if_match = Value};
+set_header(#headers{}=Hdrs, {lower, "if-match"}, Value) ->
+    Hdrs#headers{if_match = Value};
+set_header(#headers{}=Hdrs, if_none_match, Value) ->
+    Hdrs#headers{if_none_match = Value};
+set_header(#headers{}=Hdrs, {lower, "if-none-match"}, Value) ->
+    Hdrs#headers{if_none_match = Value};
+set_header(#headers{}=Hdrs, if_range, Value) ->
+    Hdrs#headers{if_range = Value};
+set_header(#headers{}=Hdrs, {lower, "if-range"}, Value) ->
+    Hdrs#headers{if_range = Value};
+set_header(#headers{}=Hdrs, if_unmodified_since, Value) ->
+    Hdrs#headers{if_unmodified_since = Value};
+set_header(#headers{}=Hdrs, {lower, "if-unmodified-since"}, Value) ->
+    Hdrs#headers{if_unmodified_since = Value};
+set_header(#headers{}=Hdrs, range, Value) ->
+    Hdrs#headers{range = Value};
+set_header(#headers{}=Hdrs, {lower, "range"}, Value) ->
+    Hdrs#headers{range = Value};
+set_header(#headers{}=Hdrs, referer, Value) ->
+    Hdrs#headers{referer = Value};
+set_header(#headers{}=Hdrs, {lower, "referer"}, Value) ->
+    Hdrs#headers{referer = Value};
+set_header(#headers{}=Hdrs, user_agent, Value) ->
+    Hdrs#headers{user_agent = Value};
+set_header(#headers{}=Hdrs, {lower, "user-agent"}, Value) ->
+    Hdrs#headers{user_agent = Value};
+set_header(#headers{}=Hdrs, accept_ranges, Value) ->
+    Hdrs#headers{accept_ranges = Value};
+set_header(#headers{}=Hdrs, {lower, "accept-ranges"}, Value) ->
+    Hdrs#headers{accept_ranges = Value};
+set_header(#headers{}=Hdrs, cookie, Value) ->
+    Hdrs#headers{cookie = Value};
+set_header(#headers{}=Hdrs, {lower, "cookie"}, Value) ->
+    Hdrs#headers{cookie = Value};
+set_header(#headers{}=Hdrs, keep_alive, Value) ->
+    Hdrs#headers{keep_alive = Value};
+set_header(#headers{}=Hdrs, {lower, "keep-alive"}, Value) ->
+    Hdrs#headers{keep_alive = Value};
+set_header(#headers{}=Hdrs, location, Value) ->
+    Hdrs#headers{location = Value};
+set_header(#headers{}=Hdrs, {lower, "location"}, Value) ->
+    Hdrs#headers{location = Value};
+set_header(#headers{}=Hdrs, content_length, Value) ->
+    Hdrs#headers{content_length = Value};
+set_header(#headers{}=Hdrs, {lower, "content-length"}, Value) ->
+    Hdrs#headers{content_length = Value};
+set_header(#headers{}=Hdrs, content_type, Value) ->
+    Hdrs#headers{content_type = Value};
+set_header(#headers{}=Hdrs, {lower, "content-type"}, Value) ->
+    Hdrs#headers{content_type = Value};
+set_header(#headers{}=Hdrs, content_encoding, Value) ->
+    Hdrs#headers{content_encoding = Value};
+set_header(#headers{}=Hdrs, {lower, "content-encoding"}, Value) ->
+    Hdrs#headers{content_encoding = Value};
+set_header(#headers{}=Hdrs, authorization, Value) ->
+    Hdrs#headers{authorization = Value};
+set_header(#headers{}=Hdrs, {lower, "authorization"}, Value) ->
+    Hdrs#headers{authorization = Value};
+set_header(#headers{}=Hdrs, transfer_encoding, Value) ->
+    Hdrs#headers{transfer_encoding = Value};
+set_header(#headers{}=Hdrs, {lower, "transfer-encoding"}, Value) ->
+    Hdrs#headers{transfer_encoding = Value};
+set_header(#headers{}=Hdrs, x_forwarded_for, Value) ->
+    Hdrs#headers{x_forwarded_for = Value};
+set_header(#headers{}=Hdrs, {lower, "x-forwarded-for"}, Value) ->
+    Hdrs#headers{x_forwarded_for = Value};
+set_header(#headers{}=Hdrs, Header, Value) when is_atom(Header) ->
+    set_header(Hdrs, atom_to_list(Header), Value);
+set_header(#headers{}=Hdrs, Header, Value) when is_binary(Header) ->
+    set_header(Hdrs, binary_to_list(Header), Value);
+set_header(#headers{}=Hdrs, Header, Val) when is_binary(Val) ->
+    set_header(Hdrs, {lower, string:to_lower(Header)}, binary_to_list(Val));
+set_header(#headers{other=Other}=Hdrs, {lower, Header}, undefined) ->
+    Handler = fun(_, true, Acc) ->
+                      Acc;
+                 (HdrVal, false, Acc) ->
+                      [HdrVal|Acc]
+              end,
+    NewOther = fold_others(Header, Handler, Other, []),
+    Hdrs#headers{other = lists:reverse(NewOther)};
+set_header(#headers{other=Other}=Hdrs, {lower, Header}, Val) ->
+    HdrName = erlang_header_name(Header),
+    Handler = fun({http_header, Int, _, Rsv, _}, true, {Acc, _}) ->
+                      {[{http_header, Int, HdrName, Rsv, Val}|Acc],true};
+                 (HdrVal, false, {Acc, Found}) ->
+                      {[HdrVal|Acc], Found}
+              end,
+    {NewOther0, Found} = fold_others(Header, Handler, Other, {[], false}),
+    NewOther = case Found of
+                   true ->
+                       NewOther0;
+                   false ->
+                       [{http_header, 0, HdrName, undefined, Val}|NewOther0]
+               end,
+    Hdrs#headers{other = lists:reverse(NewOther)};
+set_header(#headers{}=Hdrs, Header, undefined) ->
+    set_header(Hdrs, {lower, string:to_lower(Header)}, undefined);
+set_header(#headers{}=Hdrs, Header, Value) ->
+    set_header(Hdrs, {lower, string:to_lower(Header)}, Value).
+
+get_header(#headers{}=Hdrs, connection) ->
+    Hdrs#headers.connection;
+get_header(#headers{}=Hdrs, {lower, "connection"}) ->
+    Hdrs#headers.connection;
+get_header(#headers{}=Hdrs, accept) ->
+    Hdrs#headers.accept;
+get_header(#headers{}=Hdrs, {lower, "accept"}) ->
+    Hdrs#headers.accept;
+get_header(#headers{}=Hdrs, host) ->
+    Hdrs#headers.host;
+get_header(#headers{}=Hdrs, {lower, "host"}) ->
+    Hdrs#headers.host;
+get_header(#headers{}=Hdrs, if_modified_since) ->
+    Hdrs#headers.if_modified_since;
+get_header(#headers{}=Hdrs, {lower, "if-modified-since"}) ->
+    Hdrs#headers.if_modified_since;
+get_header(#headers{}=Hdrs, if_match) ->
+    Hdrs#headers.if_match;
+get_header(#headers{}=Hdrs, {lower, "if-match"}) ->
+    Hdrs#headers.if_match;
+get_header(#headers{}=Hdrs, if_none_match) ->
+    Hdrs#headers.if_none_match;
+get_header(#headers{}=Hdrs, {lower, "if-none-match"}) ->
+    Hdrs#headers.if_none_match;
+get_header(#headers{}=Hdrs, if_range) ->
+    Hdrs#headers.if_range;
+get_header(#headers{}=Hdrs, {lower, "if-range"}) ->
+    Hdrs#headers.if_range;
+get_header(#headers{}=Hdrs, if_unmodified_since) ->
+    Hdrs#headers.if_unmodified_since;
+get_header(#headers{}=Hdrs, {lower, "if-unmodified-since"}) ->
+    Hdrs#headers.if_unmodified_since;
+get_header(#headers{}=Hdrs, range) ->
+    Hdrs#headers.range;
+get_header(#headers{}=Hdrs, {lower, "range"}) ->
+    Hdrs#headers.range;
+get_header(#headers{}=Hdrs, referer) ->
+    Hdrs#headers.referer;
+get_header(#headers{}=Hdrs, {lower, "referer"}) ->
+    Hdrs#headers.referer;
+get_header(#headers{}=Hdrs, user_agent) ->
+    Hdrs#headers.user_agent;
+get_header(#headers{}=Hdrs, {lower, "user-agent"}) ->
+    Hdrs#headers.user_agent;
+get_header(#headers{}=Hdrs, accept_ranges) ->
+    Hdrs#headers.accept_ranges;
+get_header(#headers{}=Hdrs, {lower, "accept-ranges"}) ->
+    Hdrs#headers.accept_ranges;
+get_header(#headers{}=Hdrs, cookie) ->
+    Hdrs#headers.cookie;
+get_header(#headers{}=Hdrs, {lower, "cookie"}) ->
+    Hdrs#headers.cookie;
+get_header(#headers{}=Hdrs, keep_alive) ->
+    Hdrs#headers.keep_alive;
+get_header(#headers{}=Hdrs, {lower, "keep-alive"}) ->
+    Hdrs#headers.keep_alive;
+get_header(#headers{}=Hdrs, location) ->
+    Hdrs#headers.location;
+get_header(#headers{}=Hdrs, {lower, "location"}) ->
+    Hdrs#headers.location;
+get_header(#headers{}=Hdrs, content_length) ->
+    Hdrs#headers.content_length;
+get_header(#headers{}=Hdrs, {lower, "content-length"}) ->
+    Hdrs#headers.content_length;
+get_header(#headers{}=Hdrs, content_type) ->
+    Hdrs#headers.content_type;
+get_header(#headers{}=Hdrs, {lower, "content-type"}) ->
+    Hdrs#headers.content_type;
+get_header(#headers{}=Hdrs, content_encoding) ->
+    Hdrs#headers.content_encoding;
+get_header(#headers{}=Hdrs, {lower, "content-encoding"}) ->
+    Hdrs#headers.content_encoding;
+get_header(#headers{}=Hdrs, authorization) ->
+    Hdrs#headers.authorization;
+get_header(#headers{}=Hdrs, {lower, "authorization"}) ->
+    Hdrs#headers.authorization;
+get_header(#headers{}=Hdrs, transfer_encoding) ->
+    Hdrs#headers.transfer_encoding;
+get_header(#headers{}=Hdrs, {lower, "transfer-encoding"}) ->
+    Hdrs#headers.transfer_encoding;
+get_header(#headers{}=Hdrs, x_forwarded_for) ->
+    Hdrs#headers.x_forwarded_for;
+get_header(#headers{}=Hdrs, {lower, "x-forwarded-for"}) ->
+    Hdrs#headers.x_forwarded_for;
+get_header(#headers{}=Hdrs, Header) when is_atom(Header) ->
+    get_header(Hdrs, atom_to_list(Header));
+get_header(#headers{}=Hdrs, Header) when is_binary(Header) ->
+    get_header(Hdrs, binary_to_list(Header));
+get_header(#headers{other = Other}, {lower, Header}) ->
+    Handler = fun({http_header, _, _, _, Value}, true, _Acc) ->
+                      throw(Value);
+                 (_, false, Acc) ->
+                      Acc
+              end,
+    catch fold_others(Header, Handler, Other, undefined);
+get_header(#headers{}=Hdrs, Header) ->
+    get_header(Hdrs, {lower, string:to_lower(Header)}).
+
+get_header(#headers{}=Hdrs, Header, Default) ->
+    case get_header(Hdrs, Header) of
+        undefined ->
+            Default;
+        Value ->
+            Value
+    end.
+
+delete_header(#headers{}=Hdrs, Header) ->
+    set_header(Hdrs, Header, undefined).
+
+%% assumes that LowerHdr is already downcased
+fold_others(LowerHdr, Handler, Other, StartAcc) ->
+    lists:foldl(fun({http_header, _, Hdr, _, _}=HdrVal, Acc) ->
+                        HdrNm = string:to_lower(
+                                  if
+                                      is_atom(Hdr) -> atom_to_list(Hdr);
+                                      is_binary(Hdr) -> binary_to_list(Hdr);
+                                      true -> Hdr
+                                  end),
+                        Handler(HdrVal, HdrNm == LowerHdr, Acc)
+                end, StartAcc, Other).
+
+erlang_header_name("cache-control")       -> 'Cache-Control';
+erlang_header_name("date")                -> 'Date';
+erlang_header_name("pragma")              -> 'Pragma';
+erlang_header_name("upgrade")             -> 'Upgrade';
+erlang_header_name("via")                 -> 'Via';
+erlang_header_name("accept-charset")      -> 'Accept-Charset';
+erlang_header_name("accept-encoding")     -> 'Accept-Encoding';
+erlang_header_name("accept-language")     -> 'Accept-Language';
+erlang_header_name("from")                -> 'From';
+erlang_header_name("max-forwards")        -> 'Max-Forwards';
+erlang_header_name("proxy-authorization") -> 'Proxy-Authorization';
+erlang_header_name("age")                 -> 'Age';
+erlang_header_name("proxy-authenticate")  -> 'Proxy-Authenticate';
+erlang_header_name("public")              -> 'Public';
+erlang_header_name("retry-after")         -> 'Retry-After';
+erlang_header_name("server")              -> 'Server';
+erlang_header_name("vary")                -> 'Vary';
+erlang_header_name("warning")             -> 'Warning';
+erlang_header_name("www-authenticate")    -> 'Www-Authenticate';
+erlang_header_name("allow")               -> 'Allow';
+erlang_header_name("content-base")        -> 'Content-Base';
+erlang_header_name("content-encoding")    -> 'Content-Encoding';
+erlang_header_name("content-language")    -> 'Content-Language';
+erlang_header_name("content-location")    -> 'Content-Location';
+erlang_header_name("content-md5")         -> 'Content-Md5';
+erlang_header_name("content-range")       -> 'Content-Range';
+erlang_header_name("etag")                -> 'Etag';
+erlang_header_name("expires")             -> 'Expires';
+erlang_header_name("last-modified")       -> 'Last-Modified';
+erlang_header_name("set-cookie")          -> 'Set-Cookie';
+erlang_header_name("set-cookie2")         -> 'Set-Cookie2';
+erlang_header_name("proxy-connection")    -> 'Proxy-Connection';
+erlang_header_name(Name)                  -> capitalize_header(Name).
+
+capitalize_header(Name) ->
+   capitalize_header2(Name, "").
+
+capitalize_header2([C | Rest], "") when C >= $a andalso C =< $z ->
+   capitalize_header2(Rest, [C - $a + $A]);
+capitalize_header2([$-, C | Rest], Result) when C >= $a andalso C =< $z ->
+   capitalize_header2(Rest, [C - $a + $A, $- | Result]);
+capitalize_header2([C | Rest], Result) ->
+   capitalize_header2(Rest, [C | Result]);
+capitalize_header2([], Result) ->
+   lists:reverse(Result).
 
 
 reformat_request(#http_request{method = bad_request}) ->
@@ -2141,6 +2398,9 @@ bm_start(Str) ->
     Tbl = bm_set_shifts(Str, Len),
     {Tbl, list_to_binary(Str), lists:reverse(Str), Len}.
 
+bm_length({_,_,_,Len}) ->
+    Len.
+
 bm_find(Bin, SearchCtx) ->
     bm_find(Bin, SearchCtx, 0).
 bm_find(Bin, {_, _, _, Len}, Pos) when size(Bin) < (Pos + Len) ->
