@@ -1,7 +1,7 @@
 %%%----------------------------------------------------------------------
 %%% File    : yaws_compile.erl
 %%% Author  : Claes Wikstrom <klacke@hyber.org>
-%%% Purpose : 
+%%% Purpose :
 %%% Created : 20 Feb 2002 by Claes Wikstrom <klacke@hyber.org>
 %%%----------------------------------------------------------------------
 
@@ -16,8 +16,8 @@
 
 %%  tada !!
 %% returns a CodeSpec which is:
-%% a list  {data, NumChars} | 
-%%         {mod, LineNo, YawsFile, NumSkipChars,  Mod, Func} | 
+%% a list  {data, NumChars} |
+%%         {mod, LineNo, YawsFile, NumSkipChars,  Mod, Func} |
 %%         {error, NumSkipChars, E}}
 
 %% each erlang fragment inside <erl> .... </erl> is compiled into
@@ -60,16 +60,16 @@ compile_file(File) ->
             put(yfile,yaws:to_list(File))
     end,
     %% broken erlang compiler isn't
-    %% reentrant, can only have one erlang compiler at a time running 
+    %% reentrant, can only have one erlang compiler at a time running
     global:trans({yaws, self()},
                  fun() ->
                          ?Debug("Compile ~s~n", [File]),
                          case file_open(File) of
                              {ok, Fd} ->
                                  Spec = compile_file(
-                                          #comp{infile = File, 
-                                                infd = Fd, gc = GC, sc = SC}, 
-                                          1,   
+                                          #comp{infile = File,
+                                                infd = Fd, gc = GC, sc = SC},
+                                          1,
                                           get_line(), init, 0, [], 0),
                                  erase(yfile),
                                  erase(yfile_data),
@@ -94,7 +94,7 @@ clump_data([]) ->
 
 compile_file(C,  _LineNo, eof, _Mode, NumChars, Ack, Errors) ->
     file_close(C#comp.infd),
-    {ok, [{errors, Errors} | 
+    {ok, [{errors, Errors} |
           clump_data(lists:reverse([{data, NumChars} |Ack]))]};
 
 
@@ -120,36 +120,37 @@ compile_file(C, LineNo,  Chars = "<erl" ++ Tail, html,  NumChars, Ack,Es) ->
     L = length(Chars),
     if
         NumChars > 0 ->
-            compile_file(C3, LineNo+1, line() , erl,L, 
+            compile_file(C3, LineNo+1, line() , erl,L,
                          [{data, NumChars} | Ack], Es);
         true -> %% just ignore zero byte data segments
-            compile_file(C3, LineNo+1, line() , erl, L + (-NumChars), 
+            compile_file(C3, LineNo+1, line() , erl, L + (-NumChars),
                          Ack, Es) %hack
     end;
 
-compile_file(C, LineNo,  Chars = "<verbatim>" ++ _Tail, html,  
+compile_file(C, LineNo,  Chars = "<verbatim>" ++ _Tail, html,
              NumChars, Ack,Es) ->
     ?Debug("start verbatim:~p",[LineNo]),
     Len = length(Chars),
     C2 = C#comp{outfile = ["<pre>\n"]},  %% use as accumulator
-    compile_file(C2,  LineNo+1, line() , verbatim , Len, [{data, NumChars} | Ack], Es);
+    compile_file(C2,  LineNo+1, line() , verbatim , Len,
+                 [{data, NumChars} | Ack], Es);
 
-compile_file(C, LineNo,  Chars = "</verbatim>" ++ _Tail, verbatim, 
+compile_file(C, LineNo,  Chars = "</verbatim>" ++ _Tail, verbatim,
              NumChars, Ack, Es) ->
     Data = list_to_binary(lists:reverse(["</pre>\n" | C#comp.outfile])),
     Len = length(Chars),
-    compile_file(C#comp{outfile = undefined}, LineNo, line(), html, 0, 
+    compile_file(C#comp{outfile = undefined}, LineNo, line(), html, 0,
                  [{verbatim, NumChars+Len, Data} |Ack], Es);
 
 compile_file(C, LineNo,  Chars, verbatim, NumChars, Ack,Es) ->
     case has_str(Chars, ["</verbatim>"]) of
         {ok, Skipped, Chars2} ->
-            compile_file(C, LineNo,  Chars2, verbatim, 
+            compile_file(C, LineNo,  Chars2, verbatim,
                          NumChars + Skipped, Ack,Es);
         false ->
             C2 = C#comp{outfile = [yaws_api:htmlize(Chars) | C#comp.outfile]},
-            compile_file(C2, LineNo+1, line(), verbatim, NumChars + 
-                         length(Chars), Ack,Es)
+            compile_file(C2, LineNo+1, line(), verbatim, NumChars +
+                             length(Chars), Ack,Es)
     end;
 
 compile_file(C, LineNo,  _Chars = "</erl>" ++ Tail, erl, NumChars, Ack, Es) ->
@@ -159,8 +160,9 @@ compile_file(C, LineNo,  _Chars = "</erl>" ++ Tail, erl, NumChars, Ack, Es) ->
         {ok, ModuleName, Binary, Warnings} ->
             case get(use_yfile_name) of
                 true ->
-                    file:write_file("../../ebin/" ++ 
-                                    filename:rootname(C#comp.outfile)++".beam",
+                    file:write_file("../../ebin/" ++
+                                        filename:rootname(
+                                          C#comp.outfile)++".beam",
                                     Binary);
                 _ ->
                     ok
@@ -174,7 +176,7 @@ compile_file(C, LineNo,  _Chars = "</erl>" ++ Tail, erl, NumChars, Ack, Es) ->
                                  L2++[{skip, 6}|Ack], Es);
                 Err ->
                     A2 = gen_err(C, LineNo, NumChars,
-                                 ?F("Cannot load module ~p: ~p", 
+                                 ?F("Cannot load module ~p: ~p",
                                     [ModuleName, Err])),
                     compile_file(C, LineNo, Tail, html, 0,
                                  [A2, {skip, 6}|Ack], Es+1)
@@ -183,14 +185,14 @@ compile_file(C, LineNo,  _Chars = "</erl>" ++ Tail, erl, NumChars, Ack, Es) ->
             %% FIXME remove outfile here ... keep while debuging
             A2 = comp_err(C, LineNo, NumChars, Errors, Warnings),
             compile_file(C, LineNo, Tail, html, 0, [A2, {skip, 6}|Ack], Es+1);
-        {error, Str} ->  
+        {error, Str} ->
             %% this is boring but does actually happen
             %% in order to get proper user errors here we need to catch i/o
             %% or hack compiler/parser
             yaws:elog("Dynamic compile error in file ~s (~s), line ~w~n~s",
                       [C#comp.infile, C#comp.outfile,LineNo, Str]),
             A2 = {error, NumChars, ?F("<pre> Dynamic compile error in file "
-                                      " ~s line ~w~n~s </pre>", 
+                                      " ~s line ~w~n~s </pre>",
                                       [C#comp.infile, LineNo, Str])},
             compile_file(C, LineNo, Tail, html, 0, [A2, {skip, 6}|Ack], Es+1)
     end;
@@ -202,8 +204,8 @@ compile_file(C, LineNo,  Chars, erl, NumChars, Ack,Es) ->
         false ->
             ?Debug("Gen: ~s", [Chars]),
             io:format(C#comp.outfd, "~s", [Chars]),
-            compile_file(C, LineNo+1, line(), erl, NumChars + 
-                         length(Chars), Ack,Es)
+            compile_file(C, LineNo+1, line(), erl, NumChars +
+                             length(Chars), Ack,Es)
     end;
 
 compile_file(C, LineNo, [], html, NumChars, Ack, Es) ->
@@ -248,10 +250,10 @@ has_str(_,_,_) -> false.
 check_exported(C, LineNo, NumChars, Mod) ->
     case is_exported(out, 1, Mod) of
         true ->
-            [{mod, C#comp.startline, C#comp.infile, 
+            [{mod, C#comp.startline, C#comp.infile,
               NumChars,Mod,out}];
         false ->
-            ?Debug("XX ~p~n", [C]), 
+            ?Debug("XX ~p~n", [C]),
             [gen_err(C, LineNo, NumChars,
                      "out/1 is not defined ")]
     end.
@@ -314,7 +316,7 @@ new_out_file(Line, C, Tail, GC) ->
               [Line, C#comp.infile]),
     io:format(Out, "-import(yaws_api, [f/2, fl/1, postvar/2, queryvar/2])."
               " ~n~n", []),
-    io:format(Out, '-include("~s/include/yaws_api.hrl").~n', 
+    io:format(Out, '-include("~s/include/yaws_api.hrl").~n',
               [GC#gconf.yaws_dir]),
     C#comp{outfd = Out,
            outfile = OutFile}.
@@ -323,7 +325,7 @@ new_out_file(Line, C, Tail, GC) ->
 gen_err(C, _LineNo, NumChars, Err) ->
     S = io_lib:format("<p> Error in File ~s Erlang code beginning "
                       "at line ~w~n"
-                      "Error is: ~p~n", [C#comp.infile, C#comp.startline, 
+                      "Error is: ~p~n", [C#comp.infile, C#comp.startline,
                                          Err]),
     yaws:elog("~s~n", [S]),
     {error, NumChars, S}.
@@ -344,12 +346,12 @@ comp_err(C, _LineNo, NumChars, Err) ->
         [{_FileName, [ {Line0, Mod, E} |_]} |_] when is_integer(Line0) ->
             Line = Line0 + C#comp.startline - 10,
             ?Debug("XX ~p~n", [{_LineNo, Line0}]),
-            Str = io_lib:format("~s:~w:~n ~s\ngenerated file at: ~s~n", 
+            Str = io_lib:format("~s:~w:~n ~s\ngenerated file at: ~s~n",
                                 [C#comp.infile, Line,
                                  apply(Mod, format_error, [E]),
                                  C#comp.outfile
                                 ]),
-            HtmlStr = ?F("~n<pre>~nDynamic compile error: ~s~n</pre>~n", 
+            HtmlStr = ?F("~n<pre>~nDynamic compile error: ~s~n</pre>~n",
                          [Str]),
             yaws:elog("Dynamic compiler err ~s", [Str]),
             {error, NumChars,  HtmlStr};
@@ -401,7 +403,7 @@ get_compiler_data(P, Ack) ->
             {error, S};
         {P, result, {'EXIT', Reason}} ->
             S = lists:flatten(io_lib:format("~p", [Reason])),
-            {error, S} 
+            {error, S}
     end.
 
 
@@ -457,38 +459,28 @@ get_line_from_chars([H|T], Line) ->
 %% From compile.erl in order to print proper error/warning messages
 %% if compiled with check option.
 report_errors(C, Errors) ->
-    Check = true,
-    case Check of
-        true ->
-            File = "./" ++ filename:basename(C#comp.infile),
-            SLine = C#comp.startline - 10,
-            lists:foreach(fun ({{_F,_L},Eds}) -> list_errors(File, SLine, Eds);
-                              ({_F,Eds})      -> list_errors(File, SLine, Eds)
-                          end, Errors);
-        false ->
-            ok
-    end.
+    File = "./" ++ filename:basename(C#comp.infile),
+    SLine = C#comp.startline - 10,
+    lists:foreach(fun ({{_F,_L},Eds}) -> list_errors(File, SLine, Eds);
+                      ({_F,Eds})      -> list_errors(File, SLine, Eds)
+                  end, Errors).
 
 report_warnings(C, Ws0) ->
-    Check = true,
-    case Check of
-        true ->
-            File = "./" ++ filename:basename(C#comp.infile),
-            SLine = C#comp.startline - 10,
-            Ws1 = lists:flatmap(fun({{_F,_L},Eds}) ->
-                                        format_message(File, SLine, Eds);
-                                   ({_F,Eds}) ->
-                                        format_message(File, SLine, Eds)
-                                end,
-                                Ws0),
-            Ws = ordsets:from_list(Ws1),
-            lists:foreach(fun({_,Str}) -> io:put_chars(Str) end, Ws);
-        false -> ok
-    end.
+    File = "./" ++ filename:basename(C#comp.infile),
+    SLine = C#comp.startline - 10,
+    Ws1 = lists:flatmap(fun({{_F,_L},Eds}) ->
+                                format_message(File, SLine, Eds);
+                           ({_F,Eds}) ->
+                                format_message(File, SLine, Eds)
+                        end,
+                        Ws0),
+    Ws = ordsets:from_list(Ws1),
+    lists:foreach(fun({_,Str}) -> io:put_chars(Str) end, Ws).
 
 format_message(F, SLine, [{Line0,Mod,E}|Es]) ->
     Line = Line0 + SLine,
-    M = {{F,Line},io_lib:format("~s:~w: Warning: ~s\n", [F,Line,Mod:format_error(E)])},
+    M = {{F,Line},io_lib:format("~s:~w: Warning: ~s\n",
+                                [F,Line,Mod:format_error(E)])},
     [M|format_message(F, SLine, Es)];
 format_message(F, SLine, [{Mod,E}|Es]) ->
     M = {none,io_lib:format("~s: Warning: ~s\n", [F,Mod:format_error(E)])},
