@@ -1,6 +1,6 @@
 -module(app_test).
 -include("../include/tftest.hrl").
--include("../ibrowse/src/ibrowse.hrl").
+-include("../ibrowse/include/ibrowse.hrl").
 -compile(export_all).
 
 
@@ -11,7 +11,7 @@ start([F]) ->
     ibrowse:stop().
 
 start() ->
-    ?line ok,
+    io:format("\n ==== MAIN TESTS ==== \n\n", []),
     ?line {ok, _} = ibrowse:start_link(),
     server_options_test(),
     test1(),
@@ -22,12 +22,16 @@ start() ->
     sendfile_get(),
     json_test(),
     post_test(),
+    flush_test(),
     expires_test(),
     reentrant_test(),
     cgi_redirect_test(),
     php_handler_test(),
     arg_rewrite_test(),
     shaper_test(),
+    sslaccept_timeout_test(),
+    throw_test(),
+    too_many_headers_test(),
     ibrowse:stop().
 
 
@@ -573,6 +577,92 @@ large_chunked_post() ->
     ?line {ok, "200", _, _} = ibrowse:send_req(Uri, Hdrs, post, Bin, Opts2),
     ok.
 
+flush_test() ->
+    io:format("flush_test\n",[]),
+    flush_small_post(),
+    flush_large_post(),
+    flush_chunked_post(),
+    flush_small_get(),
+    flush_large_get(),
+    flush_chunked_get(),
+    ok.
+
+flush_small_post() ->
+    io:format("  flush small post\n",[]),
+    {ok, Bin} = file:read_file("../../www/1000.txt"),
+    Sz = size(Bin),
+    Uri1 = "http://localhost:8006/flushtest/" ++ integer_to_list(Sz),
+    Uri2 = "http://localhost:8006/hello.txt",
+    Hdrs = [{content_length, Sz}, {content_type, "binary/octet-stream"}],
+    {ok, ConnPid} = ibrowse:spawn_worker_process("localhost", 8006),
+    ?line {ok, "200", _, _} = ibrowse:send_req_direct(ConnPid, Uri1, Hdrs, post, Bin, []),
+    ?line {ok, "200", _, _} = ibrowse:send_req_direct(ConnPid, Uri2, [], get, [], []),
+    ibrowse:stop_worker_process(ConnPid),
+    ok.
+
+flush_large_post() ->
+    io:format("  flush large post\n",[]),
+    {ok, Bin} = file:read_file("../../www/10000.txt"),
+    Sz = size(Bin),
+    Uri1 = "http://localhost:8006/flushtest/" ++ integer_to_list(Sz),
+    Uri2 = "http://localhost:8006/hello.txt",
+    Hdrs = [{content_length, Sz}, {content_type, "binary/octet-stream"}],
+    {ok, ConnPid} = ibrowse:spawn_worker_process("localhost", 8006),
+    ?line {ok, "200", _, _} = ibrowse:send_req_direct(ConnPid, Uri1, Hdrs, post, Bin, []),
+    ?line {ok, "200", _, _} = ibrowse:send_req_direct(ConnPid, Uri2, [], get, [], []),
+    ibrowse:stop_worker_process(ConnPid),
+    ok.
+
+flush_chunked_post() ->
+    io:format("  flush chunked post\n",[]),
+    {ok, Bin} = file:read_file("../../www/10000.txt"),
+    Sz = size(Bin),
+    Uri1 = "http://localhost:8006/flushtest/chunked/" ++ integer_to_list(Sz),
+    Uri2 = "http://localhost:8006/hello.txt",
+    Hdrs = [{content_type, "binary/octet-stream"}],
+    Opts = [{transfer_encoding, {chunked, 4000*1000}}],
+    {ok, ConnPid} = ibrowse:spawn_worker_process("localhost", 8006),
+    ?line {ok, "200", _, _} = ibrowse:send_req_direct(ConnPid, Uri1, Hdrs, post, Bin, Opts),
+    ?line {ok, "200", _, _} = ibrowse:send_req_direct(ConnPid, Uri2, [], get, [], []),
+    ibrowse:stop_worker_process(ConnPid),
+    ok.
+
+flush_small_get() ->
+    io:format("  flush small get\n",[]),
+    {ok, Bin} = file:read_file("../../www/1000.txt"),
+    Sz = size(Bin),
+    Uri = "http://localhost:8006/hello.txt",
+    Hdrs = [{content_length, Sz}, {content_type, "binary/octet-stream"}],
+    {ok, ConnPid} = ibrowse:spawn_worker_process("localhost", 8006),
+    ?line {ok, "200", _, _} = ibrowse:send_req_direct(ConnPid, Uri, Hdrs, get, Bin, []),
+    ?line {ok, "200", _, _} = ibrowse:send_req_direct(ConnPid, Uri, [], get, [], []),
+    ibrowse:stop_worker_process(ConnPid),
+    ok.
+
+flush_large_get() ->
+    io:format("  flush large get\n",[]),
+    {ok, Bin} = file:read_file("../../www/10000.txt"),
+    Sz = size(Bin),
+    Uri = "http://localhost:8006/hello.txt",
+    Hdrs = [{content_length, Sz}, {content_type, "binary/octet-stream"}],
+    {ok, ConnPid} = ibrowse:spawn_worker_process("localhost", 8006),
+    ?line {ok, "200", _, _} = ibrowse:send_req_direct(ConnPid, Uri, Hdrs, get, Bin, []),
+    ?line {ok, "200", _, _} = ibrowse:send_req_direct(ConnPid, Uri, [], get, [], []),
+    ibrowse:stop_worker_process(ConnPid),
+    ok.
+
+flush_chunked_get() ->
+    io:format("  flush chunked post\n",[]),
+    {ok, Bin} = file:read_file("../../www/10000.txt"),
+    Uri = "http://localhost:8006/hello.txt",
+    Hdrs = [{content_type, "binary/octet-stream"}],
+    Opts = [{transfer_encoding, {chunked, 4000*1000}}],
+    {ok, ConnPid} = ibrowse:spawn_worker_process("localhost", 8006),
+    ?line {ok, "200", _, _} = ibrowse:send_req(Uri, Hdrs, get, Bin, Opts),
+    ?line {ok, "200", _, _} = ibrowse:send_req(Uri, [], get, [], []),
+    ibrowse:stop_worker_process(ConnPid),
+    ok.
+
 
 expires_test() ->
     io:format("expires_test\n", []),
@@ -677,6 +767,33 @@ shaper_test() ->
     ok.
 
 
+
+sslaccept_timeout_test() ->
+    io:format("sslaccept_tout_test\n", []),
+    {ok, Sock} = gen_tcp:connect("localhost", 8443, [binary, {active, true}]),
+    ?line ok = receive
+                   {tcp_closed, Sock} -> ok
+               after
+                   %% keepalive_timeout is set to 10 secs. So, wait 15 secs
+                   %% before returning an error
+                   15000 -> error
+               end,
+    gen_tcp:close(Sock),
+    ok.
+
+throw_test() ->
+    io:format("throw test\n", []),
+    Uri = "http://localhost:8009/",
+    ?line {ok, "500", _, _} = ibrowse:send_req(Uri, [], get),
+    ok.
+
+too_many_headers_test() ->
+    io:format("too many request headers test\n", []),
+    Uri = "http://localhost:8009/",
+    Hdrs = [{link, "<compact.css>; rel=\"stylesheet\"; title=\"compact\""} || _ <- lists:seq(0, 1001)],
+    ?line {ok, "431", _, _} = ibrowse:send_req(Uri, Hdrs, get),
+    ok.
+
 %% used for appmod tests
 %%
 out(_A) ->
